/*|------------------------------------------------------------------------------|
  |                              ITG Libraries KVP                               |
  |                                                                              |
  | Handles processing of Key-Value-Pair Data                                    |
  |------------------------------------------------------------------------------|*/

/*|------------------------------------------------------------------------------|
  |                         ITG Libraries KVP Functions                          |
  |                                                                              |
  | fnKVPAllKeysValid: Validate that all Keys are in the Data                    |
  | fnKVPIndexOfKey: Find index of given Key in scKVP structure                  |
  | fnKVPKeyOfIndex: Lookup Key by index in scKVP structure                      |
  | fnKVPKeysToDelmString: Format all Keys in arrKVP of scKVP structure in comma |
  |       delimited string                                                       |
  | fnKVPKeysToDelmStringEx: Format all Keys in arrKVP of scKVP structure in     |
  |       delimited string using cDelm as delimiter                              |
  | fnKVPReport: Reports KeyValuePairs in arrKVP  to telnet session              |
  | fnKVPValueOfIndex: Get value of given index  in arrKVP                       |
  | fnKVPValueOfKey: Find value of given Key in arrKVP                           |
  | fnMessageToKVP: Convert given message in KeyValuePairs and store in arrKVP   |
  |       of scKVP structure                                                     |
  | _fnCountKVP (INTERNAL FUNCTION not intended for direct usage): Count the     |
  |       number of KeyValuePairs in the passed data                             |
  |------------------------------------------------------------------------------|*/

PROGRAM_NAME='ITG Libraries KVP'
#IF_NOT_DEFINED __ITG_LIBRARIES_KVP__
#DEFINE __ITG_LIBRARIES_KVP__

#INCLUDE  'ITG Libraries Helpers'

DEFINE_TYPE

  STRUCTURE scKVP
  {
    CHAR cKey[DUET_MAX_PARAM_LEN]
    CHAR cValue[DUET_MAX_PARAM_LEN]
  }

DEFINE_VARIABLE

  scKVP arrKVP[50]

DEFINE_FUNCTION INTEGER fnKVPAllKeysValid(CHAR aKeys[][], CHAR sData[] )
{
/*|------------------------------------------------------------------------------|
  | fnKVPAllKeysValid: Validate that all Keys are in the Data                    |
  |    Parameters:                                                               |
  |       aKeys List of Keys find in data                                        |
  |       sData Data to validate Key are contained in                            |
  |    Returns:                                                                  |
  |       True if all Keys are in Data, else False                               |
  |------------------------------------------------------------------------------|*/

  STACK_VAR INTEGER idx
  STACK_VAR CHAR missingKeys[100]

  missingKeys                           =  NULL_STR

  FOR(idx =1; idx <= LENGTH_ARRAY(aKeys); idx++)
  {
    IF(fnKVPIndexOfKey(aKeys[idx]) == 0)
    {
      IF(LENGTH_STRING(missingKeys) > 0)
      {
        missingKeys                     =  fnStringConcatenate(missingKeys,',')
      }

      missingKeys                       =  fnStringConcatenate(missingKeys,aKeys[idx])
    }
  }

  IF(missingKeys == NULL_STR)
  {
    RETURN TRUE
  }

  fnIOPrintln("'fnKVPAllKeysValid missing ',missingKeys,' from ',fnKVPKeysToDelmString(),' parsed from [',sData,']'")

  RETURN FALSE
}

DEFINE_FUNCTION INTEGER fnKVPIndexOfKey(CHAR cKey[])
{
/*|------------------------------------------------------------------------------|
  | fnKVPIndexOfKey: Find index of given Key in scKVP structure                  |
  |    Parameters:                                                               |
  |       cKey Key find index of                                                 |
  |    Returns:                                                                  |
  |       Index of Key or 0 if not found                                         |
  |------------------------------------------------------------------------------|*/

  STACK_VAR INTEGER idx

  FOR(idx =1; idx <= LENGTH_ARRAY(arrKVP); idx++)
  {
    IF(fnStringIsEqualsIgnoreCase(cKey,arrKVP[idx].cKey))
    {
      RETURN idx
    }
  }

  RETURN 0
}

DEFINE_FUNCTION CHAR[STRING_RETURN_SIZE_LIMIT]  fnKVPKeyOfIndex(INTEGER index)
{
/*|------------------------------------------------------------------------------|
  | fnKVPKeyOfIndex: Lookup Key by index in scKVP structure                      |
  |    Parameters:                                                               |
  |       index Index to find Key for                                            |
  |    Returns:                                                                  |
  |       Key if found, else NULL_STR                                            |
  |------------------------------------------------------------------------------|*/

  IF(index > 0 && index <= LENGTH_ARRAY(arrKVP))
  {
    RETURN arrKVP[index].cKey
  }

  RETURN NULL_STR
}

DEFINE_FUNCTION CHAR[STRING_RETURN_SIZE_LIMIT]  fnKVPKeysToDelmString()
{
/*|------------------------------------------------------------------------------|
  | fnKVPKeysToDelmString: Format all Keys in arrKVP of scKVP structure in comma |
  |       delimited string                                                       |
  |    Returns:                                                                  |
  |       Comma delimited string of arrKVP Keys                                  |
  |------------------------------------------------------------------------------|*/

  RETURN fnKVPKeysToDelmStringEx(',')
}

DEFINE_FUNCTION CHAR[STRING_RETURN_SIZE_LIMIT]  fnKVPKeysToDelmStringEx(CHAR cDelm)
{
/*|------------------------------------------------------------------------------|
  | fnKVPKeysToDelmStringEx: Format all Keys in arrKVP of scKVP structure in     |
  |       delimited string using cDelm as delimiter                              |
  |    Parameters:                                                               |
  |       cDelm delimiter                                                        |
  |    Returns:                                                                  |
  |       Delimited string of arrKVP Keys                                        |
  |------------------------------------------------------------------------------|*/

  STACK_VAR INTEGER idx
  STACK_VAR CHAR sData[STRING_RETURN_SIZE_LIMIT]

  FOR(idx = 1; idx <= LENGTH_ARRAY(arrKVP); idx++)
  {
    IF(idx > 1)
    {
      sData                             =  fnStringConcatenate(sData,cDelm)
    }

    sData                               =  fnStringConcatenate(sData,arrKVP[idx].cKey)
  }

  RETURN sData
}

DEFINE_FUNCTION fnKVPReport()
{
/*|------------------------------------------------------------------------------|
  | fnKVPReport: Reports KeyValuePairs in arrKVP  to telnet session              |
  |------------------------------------------------------------------------------|*/

  STACK_VAR INTEGER idx
  STACK_VAR INTEGER longKeyLen
  STACK_VAR CHAR fmttedKey[DUET_MAX_PARAM_LEN]

  longKeyLen                            =  1

  FOR(idx =1; idx <= LENGTH_ARRAY(arrKVP); idx++)
  {
    IF(LENGTH_STRING(arrKVP[idx].cKey) > longKeyLen)
    {
      longKeyLen                        =  LENGTH_STRING(arrKVP[idx].cKey)
    }
  }

  fnIOPrintln(' ')

  fnIOPrintln('Message as KVP >>>>>>>>>>>>>>>>>>>>')

  FOR(idx =1; idx <= LENGTH_ARRAY(arrKVP); idx++)
  {
    fmttedKey                           =  fnStringSuffixToLength(arrKVP[idx].cKey,' ',longKeyLen)

    IF(LENGTH_ARRAY(arrKVP) > 9 && idx < 10)
    {
      fnIOPrintln("$09,ITOA(idx),'  ',fmttedKey,': ',arrKVP[idx].cValue")
    }
    ELSE
    {
      fnIOPrintln("$09,ITOA(idx),' ',fmttedKey,': ',arrKVP[idx].cValue")
    }
  }

  fnIOPrintln('<<<<<<<<<<<<<<<<<<<< Message as KVP')
  fnIOPrintln(' ')
}

DEFINE_FUNCTION CHAR[STRING_RETURN_SIZE_LIMIT]  fnKVPValueOfIndex(INTEGER index)
{
/*|------------------------------------------------------------------------------|
  | fnKVPValueOfIndex: Get value of given index  in arrKVP                       |
  |    Parameters:                                                               |
  |       index Index of KVP to find value of                                    |
  |    Returns:                                                                  |
  |       Value if found, else NULL_STR                                          |
  |------------------------------------------------------------------------------|*/

  IF(index > 0 && index <= LENGTH_ARRAY(arrKVP))
  {
    RETURN arrKVP[index].cValue
  }

  RETURN NULL_STR
}

DEFINE_FUNCTION CHAR[STRING_RETURN_SIZE_LIMIT]  fnKVPValueOfKey(CHAR cKey[])
{
/*|------------------------------------------------------------------------------|
  | fnKVPValueOfKey: Find value of given Key in arrKVP                           |
  |    Parameters:                                                               |
  |       cKey Key to find value of                                              |
  |    Returns:                                                                  |
  |       Value if found, else NULL_STR                                          |
  |------------------------------------------------------------------------------|*/

  RETURN fnKVPValueOfIndex(fnKVPIndexOfKey(cKey))
}

DEFINE_FUNCTION fnMessageToKVP(CHAR sPassedData[])
{
/*|------------------------------------------------------------------------------|
  | fnMessageToKVP: Convert given message in KeyValuePairs and store in arrKVP   |
  |       of scKVP structure                                                     |
  |    Parameters:                                                               |
  |       sPassedData Voice agent message received                               |
  |------------------------------------------------------------------------------|*/

  STACK_VAR CHAR sTmp[DUET_MAX_PARAM_LEN]
  STACK_VAR CHAR cCmd[DUET_MAX_CMD_LEN]
  STACK_VAR INTEGER idx
  STACK_VAR CHAR sData[DUET_MAX_CMD_LEN]

  SET_LENGTH_ARRAY(arrKVP, _fnCountKVP(sPassedData))

  sData                                 =  sPassedData
  idx                                   =  0

  WHILE(LENGTH_STRING(sData) > 0)
  {
    sTmp                                =  fnStringReplace(DuetParseCmdParam(sData),'=','-')
    cCmd                                =  sTmp
    idx++

    arrKVP[idx].cKey                    =  DuetParseCmdHeader(sTmp)
    arrKVP[idx].cValue                  =  sTmp
  }
}

DEFINE_FUNCTION INTEGER _fnCountKVP(CHAR sPassedData[])
{
/*|------------------------------------------------------------------------------|
  | _fnCountKVP (INTERNAL FUNCTION not intended for direct usage): Count the     |
  |       number of KeyValuePairs in the passed data                             |
  |    Parameters:                                                               |
  |       sPassedData Voice agent message received                               |
  |    Returns:                                                                  |
  |       Count of KeyValuePairs                                                 |
  |------------------------------------------------------------------------------|*/

  STACK_VAR INTEGER cnt
  STACK_VAR CHAR sData[DUET_MAX_CMD_LEN]

  sData                                 =  sPassedData
  cnt                                   =  0

  WHILE(LENGTH_STRING(sData) > 0)
  {
    fnStringReplace(DuetParseCmdParam(sData),'=','-')
    cnt++
  }

  RETURN cnt
}

DEFINE_PROGRAM

/*|------------------------------------------------------------------------------|
  |                               !!! WARNING !!!                                |
  |                                                                              |
  | Due to differences in the underlying architecture of the X-Series masters,   |
  |       changing variables in the DEFINE_PROGRAM section of code can           |
  |       negatively impact program performance.                                 |
  |                                                                              |
  | TimeLine implementation refer to "Differences in DEFINE_PROGRAM Program      |
  |       Execution" section of the NX-Series Controllers WebConsole &           |
  |       Programming Guide for additional and alternate coding methodologies.   |
  |------------------------------------------------------------------------------|*/

  #END_IF  // __ITG_LIBRARIES_KVP__

/*|------------------------------------------------------------------------------|
  |                                                                              |
  |      !!! END OF PROGRAM --- DO NOT PUT ANY CODE BELOW THIS COMMENT  !!!      |
  |                                                                              |
  |------------------------------------------------------------------------------|*/
